########## S script: sec.10.1.s ##########

# S-PLUS code listed in Section 10.1 of
# the paper "Smoothing and Mixed Model Software"
# by Ngo & Wand.

# Load in default.knots() function.
	
source("default.knots.sf")

fossil <- read.table("fossil.dat",header=T)
x <- fossil$age
y <- 100000*fossil$strontium.ratio

# Set the value of the smoothing parameter (variance ratio)

alpha <- 520

# Set up design matrices, for linear splines
# in this case.

n <- length(x)
X <- cbind(rep(1,n),x)
knots <- default.knots(x)
Z <- outer(x,knots,"-")
Z <- Z*(Z>0)

# Set up input matrices for Algorithm A.1.

C.mat <- cbind(X,Z)
CTC <- t(C.mat)%*%C.mat
D.mat <- diag(c(rep(0,ncol(X)),rep(1,ncol(Z))))

# Carry out Steps 1 and 2 of Algorithm A.1.

R.mat <- chol(CTC)
svd.out <- svd(t(solve(t(R.mat),
           t(solve(t(R.mat),D.mat)))))
s.vec <- svd.out$d
U.mat <- svd.out$u
A.mat <- C.mat%*%backsolve(R.mat,U.mat)
b.vec <- as.vector(t(A.mat)%*%y)

# Obtain vector of fitted values.

f.hat <- A.mat%*%(b.vec/(1+alpha*s.vec))
df.fit <- sum(1/(1+alpha*s.vec))

# Get the variability var 

RSS <- sum((y - f.hat)^2)
r.vec <- 1/(1+alpha*s.vec)
df.fit <- sum(r.vec)
df.res <- n - 2*df.fit + sum(r.vec^2)
sig.eps.hat <- sqrt(RSS/df.res)
st.dev.hat <- sig.eps.hat*sqrt(diag(A.mat%*%(r.vec*t(A.mat))))
var.bar.upp <- f.hat + 2*st.dev.hat
var.bar.low <- f.hat - 2*st.dev.hat

########## End of sec.10.1.s ##########
