########## S script: sec.11.1.s ##########

# S-PLUS code listed in Section 11.1 of
# the paper "Smoothing and Mixed Model Software"
# by Ngo & Wand.


fossil <- read.table("fossil.dat",header=T)
x <- fossil$age
y <- 100000*fossil$strontium.ratio
knots <- seq(94,121,length=25)
n <- length(x)
X <- cbind(rep(1,n),x)

svd.Omega <- svd((abs(outer(knots,knots,"-")))^3)
matrix.sqrt.Omega <- svd.Omega$u%*%sqrt(diag(svd.Omega$d))%*%t(svd.Omega$v)
Z <- ((abs(outer(x,knots,"-")))^3)%*%solve(matrix.sqrt.Omega)

fit <- lme(y~-1+X,random=pdIdent(~-1+Z))
beta.hat <- fit$coef$fixed
u.hat <- unlist(fit$coef$random)
f.hat <- X%*%beta.hat + Z%*%u.hat
sig.eps.hat <- fit$sigma
sig.u.hat   <- sig.eps.hat*exp(unlist(fit$modelStruct))

# Plotting the data and smoothed curve

num.grid <- 401
x.grid <- seq(min(x),max(x),length=num.grid)
X.grid <- cbind(rep(1,num.grid),x.grid)
Z.grid <- ((abs(outer(x.grid,knots,"-")))^3)%*%solve(matrix.sqrt.Omega)
fhat.grid <- X.grid%*%beta.hat + Z.grid%*%u.hat
plot(x,y,pch=1)
lines(x.grid,fhat.grid)

########## End of sec.11.1.s ##########