/********** SAS code: sec.11.2.sas **************

 SAS code listed in Section 11.2 of
 the paper "Smoothing and Mixed Model Software"
 by Ngo & Wand
 
 ***********************************************/

options ls=132; 
libname paper '~/test';
data paper.fossil;
  infile '~/test/fossil.dat' missover;
  input age ratio;
  ratio=ratio*100000;
  if age ne .;
run;  

/*********************************/
/* calling macro to create knots */
/*********************************/  

%include "default_knots.macro";
%default_knots(librefknots=paper,data=paper.fossil,knotdata=knots,varknots=age);
data dataw;
  set paper.fossil;
  m=1;
run;

data kt1;
  set paper.knots nobs=nk;
  call symput('nkt',nk);
  m=1;
run;
proc transpose data=paper.knots prefix=knots out=knotst;
  var knots;
run;
data paper.knotst;
  set knotst;
  m=1;
run;

/***********************************/
/* creating the Z(k) matrix        */
/***********************************/ 

data Zk;
  merge dataw paper.knotst;
  by m;
  %let nk=&nkt;
  array Z (&nk) Z1-Z&nk;
  array knots (&nk) knots1-knots&nk;
  do k=1 to &nk;
     Z(k)=(abs(age-knots(k)))**3;       
  end;
  keep Z1-Z&nk;
run;

/***********************************/
/* creating the O(k) matrix        */
/***********************************/

data Ok;
  merge kt1 paper.knotst;
  by m;
  %let nk=&nkt;
  array O (&nk) O1-O&nk;
  array knotsa (&nk) knots1-knots&nk;
  do k=1 to &nk;
     O(k)=(abs(knots-knotsa(k)))**3;   
  end;
  keep O1-O&nk;
run;

/***********************************/
/* creating the Z matrix           */
/***********************************/

proc iml;
  use Zk;
  read all var _num_ into Zk;
  use Ok;
  read all var _num_ into Ok;
  call svd(u,d,v,Ok);
  sqrtOk=u*sqrt(diag(d))*v`;
  Z=Zk*inv(sqrtOk);
    create Z from Z[colname={col1 col2 col3 col4 col5 col6 col7 
                           col8 col9 col10 col11 col12 
                           col13 col14 col15 col16 col17 col18 
                           col19 col20 col21}];
  append from Z;
quit;
run; 
 
data dataw2;
  merge dataw Z;
run;

ods output CovParms=paper.varcomp;

/********************************/
/* fitting the mixed model      */
/********************************/ 

proc mixed;
  model ratio = age / solution outp=paper.yhat;
  random  COL1-COL&nk / type=toep(1) s;
run;

/********************************/
/* plotting the smoothed curve  */
/********************************/

proc sort;
  by age;
run;
symbol1 v=circle c=black i=j l=1;
symbol2 v=point  c=blue  i=j l=2;
goptions device=xcolor;
proc gplot;
  plot ratio*age pred*age / overlay;
run;   

************* End of sec.11.2.sas **************