########## S script: sec.12.1.s ##########

# S-PLUS code listed in Section 12.1 of
# the paper "Smoothing and Mixed Model Software"
# by Ngo & Wand.

tps.cov <- function(r)
{     
    r <- as.matrix(r)
    num.row <- nrow(r)
    num.col <- ncol(r)
    r <- as.vector(r)
    nzi <- (1:length(r))[r!=0]
    ans <- rep(0,length(r))
    ans[nzi] <- r[nzi]^2*log(abs(r[nzi]))
    if (num.col>1) ans <- matrix(ans,num.row,num.col)

    return(ans)
}

# Set the point cloud variables to be smoothed.
 
scallop <- read.table("scallop.dat",header=T)
x1 <- scallop$long
x2 <- scallop$lat
y <- log(scallop$tcatch + 1)

# Read in the knots from a file. These
# were created using a space-filling algori`thm.

knots <- as.matrix(read.table("scallop.knots",header=T))
K <- nrow(knots)

# Set up the design matrices corresponding
# to a plane for X and thin plate spline basis functions for Z

X <- cbind(rep(1,length(y)),x1,x2)

dist.mat <- matrix(0,K,K)
dist.mat[lower.tri(dist.mat)] <- dist(knots)
dist.mat <- dist.mat + t(dist.mat)

Omega <- tps.cov(dist.mat)

diffs.1 <- outer(x1,knots[,1],"-")
diffs.2 <- outer(x2,knots[,2],"-")
dists <- sqrt(diffs.1^2+diffs.2^2)

svd.Omega <- svd(Omega)
sqrt.Omega <- t(svd.Omega$v %*% (t(svd.Omega$u) * sqrt(svd.Omega$d)))
Z <- t(solve(sqrt.Omega,t(tps.cov(dists))))

# Obtain the bivariate smooth using lme()
# and extract the coefficients.

fit <- lme(y~-1+X,random=pdIdent(~-1+Z))

beta.hat <- fit$coef$fixed
u.hat <- unlist(fit$coef$random)

########## End of sec.12.1.s ##########
