/********** SAS code: sec.12.2.sas **************

 SAS code listed in Section 12.2 of
 the paper "Smoothing and Mixed Model Software"
 by Ngo & Wand
 
 ***********************************************/

options ls=132; 
libname paper '~/test';
data paper.scallop;
  infile '~/test/scallop.dat' missover;
  input strata sample lat long tcatch prerec recruits;
  y=log(tcatch+1);
  m=1;
  if strata ne .;
  keep y lat long m;
run; 
/*******************************************************/ 
/*Read in the knots data -- as used in the Splus module*/
/*******************************************************/
data knots;
  infile '~/test/scallop.knots' missover;
  input x1 x2;
  if x1 ne .;
run; 
data knots;
  set knots nobs=nk;
  call symput('nkt',nk);
run;
%let numknots=&nkt;
proc transpose out=t1;
  var x1 x2;
run;
/********************************************************/
/*Compute the matrix Omega                              */
/********************************************************/
data d1 (keep=i j xt1) d2 (keep=xt2);
  set t1;
  array da (&numknots) col1-col&numknots; 
  do i=1 to &numknots-1;
     do j=i+1 to &numknots;
        if _name_='x1' then do;
           xt1=(da(j)-da(i))**2;
           output d1;
        end; else 
        if _name_='x2' then do;
           xt2=(da(j)-da(i))**2;
           output d2;
        end;
      end;
   end;
run;
data e1;
  merge d1 d2;
  dist=sqrt(xt1+xt2);
  omegaelm=dist*dist*log(dist);
  keep i j omegaelm;
run;
/********************************************************/
/*Construct the Zk matrix                               */
/********************************************************/
data t1a;
  set t1;
  if _name_='x1';
  m=1;
  drop _name_;
run;
data diffs1;
  merge paper.scallop (keep=long m) t1a;
  by m;
  array cola (&numknots) col1-col&numknots;
  array z1a  (&numknots) z1_1-z1_&numknots;
  do i=1 to &numknots;
     z1a(i)=long-cola(i);
  end; 
  keep z1_1-z1_&numknots;
run;
data t2a;
  set t1;
  if _name_='x2';
  m=1;
  drop _name_;
run;
data diffs2;
  merge paper.scallop (keep=lat m) t2a;
  by m;
  array cola (&numknots) col1-col&numknots;
  array z2a  (&numknots) z2_1-z2_&numknots;
  do i=1 to &numknots;
     z2a(i)=lat-cola(i);
  end; 
  keep z2_1-z2_&numknots;
run;
data dists;
  merge diffs1 diffs2;
  array z1a (&numknots) z1_1-z1_&numknots;
  array z2a (&numknots) z2_1-z2_&numknots;
  array dista (&numknots) dist1-dist&numknots;
  do i=1 to &numknots;
     temp1=sqrt(z1a(i)**2+z2a(i)**2);
     if temp1=0 then dista(i)=0; else
     dista(i)=(temp1**2)*log(temp1);
  end; 
  keep dist1-dist&numknots;
run;
/**********************************************************/
/*Construct the Z matrix from the Omega and Zk matrix     */
/**********************************************************/
proc iml;
  use e1;
  read all var _num_ into e1;
  omega=j(&numknots,&numknots,0);
  do i=1 to (&numknots*(&numknots-1))/2;
    omega[e1[i,1],e1[i,2]]=e1[i,3];
    omega[e1[i,2],e1[i,1]]=e1[i,3];
  end;
  call svd(u,d,v,omega);
  sqrtomega=u*sqrt(diag(d))*v`;
  use dists;
  read all var _num_ into Zk;
  Z=Zk*inv(sqrtomega);
  create Z from Z[colname={col1 col2 col3 col4 col5 col6 col7 
                           col8 col9 col10 col11 col12 
                           col13 col14 col15 col16 col17 col18 
                           col19 col20 col21 col22 col23 col24 col25
                           col26 col27 col28 col29 col30 col31 col32
                           col33 col34 col35 col36 col37 col38 col39
                           col40 col41 col42 col43 col44 col45 col46
                           col47 col48}];
  append from Z;
quit;
data dataw2;
  merge paper.scallop Z;
run;
ods output CovParms=paper.varcomp;
/********************************/
/* fitting the mixed model      */
/********************************/ 
proc mixed;
  model y = long lat / solution outp=paper.yhat;
  random  col1-col&numknots / type=toep(1) s;
run;

************* End of sec.12.2.sas **************
