########## S script: sec.13.1.s ##########

# S-PLUS code listed in Section 13.1 of
# the paper "Smoothing and Mixed Model Software"
# by Ngo & Wand.

# Load in default.knots() function.
	
source("default.knots.sf")	

# Read in data and assign regression vectors and knots:

trade.union <- read.table("tradeunion.dat",header=T)

x <- trade.union$wage
y <- trade.union$union.member

knots <- default.knots(x)
n <- length(y)

# Set the smoothing parameter:


alpha <- 1000

# Set the design matrices for quadratic penalized splines:


X <- cbind(rep(1,n),x,x^2)

Z <- outer(x,knots,"-")
Z <- Z*(Z>0)
Z <- Z^2

C.mat <- cbind(X,Z) 
D.mat <- diag(c(rep(0,ncol(X)),rep(1,ncol(Z))))

# Find an initial estimate based on an ordinary
# ridge regression fit using Algorithm 1:


svd.C <- svd(C.mat)

U.C <- svd.C$u
V.C <- svd.C$v
d.C <- svd.C$d

svd.D <- svd(t(t(t(V.C)%*%D.mat%*%V.C/d.C)/d.C))

d.D <- svd.D$d

A.mat <- U.C%*%svd.D$u  

b.vec <- as.vector(t(A.mat)%*%y)



eta.hat <- A.mat%*%(b.vec/(1+alpha*d.D))

# Now do iteratively reweighted penalized fits


desired.accuracy <- 0.001

rel.error <- desired.accuracy+1
max.iter <- 100

iter.num <- 0
while((rel.error>desired.accuracy)&(iter.num<max.iter))
{
   eta.hat.old <- eta.hat
  
   wt.vec <- as.vector(exp(eta.hat)/((1+exp(eta.hat))^2))
   Cw.mat <- C.mat*sqrt(wt.vec)
   y.adj <- eta.hat + (y-eta.hat)/wt.vec
 
   svd.C <- svd(Cw.mat)

   U.C <- svd.C$u
   V.C <- svd.C$v
   d.C <- svd.C$d

   svd.D <- svd(t(t(t(V.C)%*%D.mat%*%V.C/d.C)/d.C))
   d.D <- svd.D$d

   A.mat <- U.C%*%svd.D$u/sqrt(wt.vec)
   b.vec <-  as.vector(t(A.mat)%*%(wt.vec*y.adj))

   eta.hat <- A.mat%*%(b.vec/(1+alpha*d.D))

   rel.error <- sum(abs(eta.hat-eta.hat.old))/sum(abs(eta.hat.old))

   iter.num <- iter.num + 1  
}

# Fitted probabilities are in the vector eta.hat

########## End of sec.13.1.s ##########
