/********** SAS code: sec.13.2.sas **************

 SAS code listed in Section 13.2 of
 the paper "Smoothing and Mixed Model Software"
 by Ngo & Wand
 
 ***********************************************/

options ls=132; 
libname paper '~/test';
data paper.tradeunion;
  infile '~/test/tradeunion.dat' missover;
  input yearedu south female yearsexpe member wage age race occupation sector
        married;
  m=1;
  wage2=wage**2;
  if member ne .; 
  keep wage wage2 member m;
run; 
/*********************************************/
/* creating knots for the smoothing variable */
/*********************************************/

options mprint; 

%include "default_knots.macro"; 
%default_knots(librefknots=paper,data=paper.tradeunion,knotdata=knots,varknots=wage,numknots=35);

data dataw;
  set paper.tradeunion;
  m=1;
run;
data kt;
  set paper.knots nobs=nk;
  call symput('nkt',nk);
run;
proc transpose data=paper.knots prefix=knots out=knotst;
  var knots;
run;

data paper.knotst;
  set knotst;
  m=1;
run;


/***********************************/
/* creating the Z matrix           */
/***********************************/

data dataw;
  merge dataw paper.knotst;
  by m;
  %let nk=&nkt;
  array Z  (&nk) Z1-Z&nk;
  array knots (&nk) knots1-knots&nk;
  do k=1 to &nk;
     Z(k)=wage-knots(k);
     if Z(k) < 0 then Z(k)=0;
     Z(k)=Z(k)**2;
  end;
  drop knots1-knots&nk _name_;
run;
/**********************************************/
/*Fit generalized linear mixed model          */
/**********************************************/
*note that the statement Z1-Z&nk does not work here in glimmix;
%include 'glimmix.sas';
%glimmix(data=dataw,procopt=method=reml,
         stmts=%str(
                    model member=wage wage2 / solution;
                    random Z1 Z2 Z3 Z4 Z5 Z6 Z7 Z8 Z9 Z10
                     Z11 Z12 Z13 Z14 Z15 Z16 Z17 Z18 Z19 Z20
                     Z21 Z22 Z23 Z24 Z25 Z26 Z27 Z28 Z29 Z30
                     Z31 Z32 Z33 Z34 Z35
                     / type=toep(1) s;),
                    error=binomial,
                    link=logit,
                    out=fitted);

proc print data=fitted; 
var mu;
title 'Fitted Probabilities'; 
run;

************* End of sec.13.2.sas **************
