########## S script: sec.5.1.s ##########

# S-PLUS code listed in Section 5.1 of
# the paper "Smoothing and Mixed Model Software"
# by Ngo & Wand.

# Load in default.knots() function.
	
source("default.knots.sf")	

milanmort <- read.table("milanmort.dat",header=T)
year.num <- 1
subinds <- (365*(year.num-1)+1):(365*year.num)
milanmort <- milanmort[subinds,]
y <- sqrt(milanmort$resp.mort)
x.1 <- milanmort$day.num
x.2 <- milanmort$mean.temp
x.3 <- milanmort$rel.humid
x.4 <- milanmort$TSP

# Set up design matrices

X <- cbind(rep(1,length(y)),x.1,x.2,x.3,x.4)
knots.1 <- default.knots(x.1)
Z.1 <- outer(x.1,knots.1,"-")
Z.1 <- Z.1*(Z.1>0)
K.1 <- length(knots.1)
knots.2 <- default.knots(x.2)
Z.2 <- outer(x.2,knots.2,"-")
Z.2 <- Z.2*(Z.2>0)
K.2 <- length(knots.2)
knots.3 <- default.knots(x.3)
Z.3 <- outer(x.3,knots.3,"-")
Z.3 <- Z.3*(Z.3>0)
K.3 <- length(knots.3)

Z <- cbind(Z.1,Z.2,Z.3)


# Fit the additive model using 
# First the block structure of the
# random effects covariance matrix
# must be specified and stored in the list 

re.block.inds <- list(1:K.1,(K.1+1):(K.1+K.2),
                     (K.1+K.2+1):(K.1+K.2+K.3))         
Z.block  <-  list()
for (i in 1:length(re.block.inds))
   Z.block[[i]] <- as.formula(paste("~Z[,c(",paste(
             re.block.inds[[i]],collapse=","),")]-1"))

fit <- lme(y~-1+X,random=pdBlocked(Z.block,pdClass="pdIdent"))
beta.hat <- fit$coef$fixed
u.hat <- unlist(fit$coef$random)

# Extract the estimated variance components.

sig.eps.hat <- fit$sigma
sig.u.hat  <- sig.eps.hat*exp(unlist(fit$modelStruct))

# Print summary of fixed effects

print(summary(fit)$tTable)

########## End of sec.5.1.s ##########