/********** SAS code: sec.5.2.sas **************

 SAS code listed in Section 5.2 of
 the paper "Smoothing and Mixed Model Software"
 by Ngo & Wand
 
 ***********************************************/

libname paper '~/test';
data paper.milan1;
  infile '~/test/milanmort.dat' missover;
  input daynum dayweek holiday meantemp relhumid totmort respmort s02 tsp;
  y=sqrt(respmort);
  x1=daynum;
  x2=meantemp;
  x3=relhumid;
  x4=tsp;
  if daynum ne . ; 
run;
data paper.milan2;
  set paper.milan1; 
  if _n_ <= 365;
run;

/********************************************/
/* creating knots for 3 smoothing variables */
/********************************************/

%include "default_knots.macro"; 
%default_knots(librefknots=paper,data=paper.milan2,knotdata=knots1,varknots=x1);
%default_knots(librefknots=paper,data=paper.milan2,knotdata=knots2,varknots=x2);
%default_knots(librefknots=paper,data=paper.milan2,knotdata=knots3,varknots=x3);

data dataw;
  set paper.milan2 (keep=y x1-x4);
  m=1;
run;
data kt1;
  set paper.knots1 nobs=nk1;
  call symput('nkt1',nk1);
run;
proc transpose data=paper.knots1 prefix=knots1_ out=knotst1;
  var knots;
run;
data kt2;
  set paper.knots2 nobs=nk2;
  call symput('nkt2',nk2);
run;
proc transpose data=paper.knots2 prefix=knots2_ out=knotst2;
  var knots;
run;
data kt3;
  set paper.knots3 nobs=nk3;
  call symput('nkt3',nk3);
run;
proc transpose data=paper.knots3 prefix=knots3_ out=knotst3;
  var knots;
run;
data paper.knotst;
  merge knotst1 knotst2 knotst3;
  m=1;
run;

/***********************************/
/* creating the Z matrix           */
/***********************************/

data dataw;
  merge dataw paper.knotst;
  by m;
  %let nk1=&nkt1;
  %let nk2=&nkt2;
  %let nk3=&nkt3;
  array Z1a (&nk1) Z1_1-Z1_&nk1;
  array knots1a (&nk1) knots1_1-knots1_&nk1;
  do k=1 to &nk1;
     Z1a(k)=x1-knots1a(k);
     if Z1a(k) < 0 then Z1a(k)=0;
  end;
  array Z2a (&nk2) Z2_1-Z2_&nk2;
  array knots2a (&nk2) knots2_1-knots2_&nk2;
  do k=1 to &nk2;
     Z2a(k)=x2-knots2a(k);
     if Z2a(k) < 0 then Z2a(k)=0;
  end;
  array Z3a (&nk3) Z3_1-Z3_&nk3;
  array knots3a (&nk3) knots3_1-knots3_&nk3;
  do k=1 to &nk3;
     Z3a(k)=x3-knots3a(k);
     if Z3a(k) < 0 then Z3a(k)=0;
  end;
  drop knots1_1-knots1_&nk1 knots2_1-knots2_&nk2 knots3_1-knots3_&nk3 _name_;
run;
ods output CovParms=paper.varcomp;

/************************************/
/* fitting the additive model       */
/************************************/

proc mixed;
  model y = x1-x4 / solution outp=paper.yhat;
  random  Z1_1-Z1_&nk1 / type=toep(1) s;
  random  Z2_1-Z2_&nk2 / type=toep(1) s;
  random  Z3_1-Z3_&nk3 / type=toep(1) s;
run;
 
************* End of sec.5.2.sas ************** 
