########## S script: sec.6.1.s ##########

# S-PLUS code listed in Section 6.1 of
# the paper "Smoothing and Mixed Model Software"
# by Ngo & Wand.

sitka <- read.table("sitka_spruce.dat",header=T)
idnum <- sitka$idnum
order <- sitka$order
days <- sitka$days
log.size <- sitka$log.size
ozone <- sitka$ozone

# Construct the response vector and X matrix

y <- log.size
X <- cbind(rep(1,length(y)),days,ozone)

# Create the spline component of the Z matrix. Note that
# the presence of  knots for the #days variable can be assumed

knots <- default.knots(days)

Z.spline <- outer(days,knots,"-")
Z.spline <- Z.spline*(Z.spline>0)         

# The component of the Z matrix corresponding to the random
# intercept needs to be constructed with dummy variables

id <- factor(idnum)

fit <- lme(y~-1+X,random=pdBlocked(list(pdIdent(~-1+id),
                      pdIdent(~-1+Z.spline))))

beta.hat <- fit$coef$fixed
u.hat <- unlist(fit$coef$random)

sig.eps.hat <- fit$sigma
sig.u.hat  <- sig.eps.hat*exp(2*unlist(fit$modelStruct))
print(summary(fit)$tTable)

########## End of sec.6.1.s ##########