/********** SAS code: sec.6.2.sas **************

 SAS code listed in Section 6.2 of
 the paper "Smoothing and Mixed Model Software"
 by Ngo & Wand
 
 ***********************************************/

libname paper '~/test';
data paper.sitka1;
  infile '~/test/sitka_spruce.dat' missover;
  input idnum order days logsize ozone;
  if idnum ne .; 
run;  

/*********************************************/
/* Creating knots for the smoothing variable:*/
/* these knots were obtained from the Splus  */
/* program sec6.1.s. The fixed effects       */
/* estimates are thus identical to those of  */
/* the Splus code.                           */      
/*********************************************/
data paper.knots;
   input knots;
datalines;
196.5
247.6667
498.5
563.6667
617.3333
;
run;

data dataw;
  set paper.sitka1 (keep=idnum logsize days ozone);
  m=1;
run;

data paper.knots;
   input knots;
datalines;
196.5
247.6667
498.5
563.6667
617.3333
;
run;

data kt;
  set paper.knots nobs=nk;
  call symput('nkt',nk);
run;
proc transpose data=paper.knots prefix=knots out=knotst;
  var knots;
run;

data paper.knotst;
  set knotst;
  m=1;
run;


/***********************************/
/* creating the Z matrix           */
/***********************************/

data dataw;
  merge dataw paper.knotst;
  by m;
  %let nk=&nkt;
  array Z (&nk) Z1-Z&nk;
  array knots (&nk) knots1-knots&nk;
  do k=1 to &nk;
     Z(k)=days-knots(k);
     if Z(k) < 0 then Z(k)=0;
  end;
  drop knots1-knots&nk _name_;
run;
ods output CovParms=paper.varcomp;

/************************************/
/* fitting the additive model       */
/************************************/

proc mixed;
  class idnum;
  model logsize = days ozone / solution outp=paper.yhat;
  random idnum / type=toep(1) s;
  random  Z1-Z&nk / type=toep(1) s;
run;
 
************* End of sec.6.2.sas **************