########## S script: sec.7.1.s ##########

# S-PLUS code listed in Section 7.1 of
# the paper "Smoothing and Mixed Model Software"
# by Ngo & Wand.


   milan.mort <- read.table("milanmort.dat",header=T)

   # Set up variables

   n <- 4*365
   holiday <- milan.mort$holiday[1:n]
   temperature <- milan.mort$mean.temp[1:n]
   sqrt.mort <- sqrt(milan.mort$resp.mort)[1:n]
   TSP <- milan.mort$TSP[1:n]
   rel.humid <- milan.mort$rel.humid[1:n]

   # Create "day in season" variable

   day.in.seas <- rep(1:365,4)

   # Set up the knots

   K <- 15
   knots <- quantile(unique(day.in.seas),seq(0,1,length=K+2))[-c(1,K+2)]

   inds.1984 <- (1:365)
   inds.1985 <- 365 + (1:365) 
   inds.1986 <- 2*365 + (1:365) 
   inds.1987 <- 3*365 + (1:365) 

   # Now fit using lme()
   
   # Determine dummies
   
   indic.1984 <- rep(0,n)
   indic.1985 <- rep(0,n) 
   indic.1986 <- rep(0,n)
   indic.1987 <- rep(0,n)
   indic.1984[inds.1984] <- 1
   indic.1985[inds.1985] <- 1
   indic.1986[inds.1986] <- 1
   indic.1987[inds.1987] <- 1
   
   # Set up X matrix
   
   X <- cbind(rep(1,n),holiday,day.in.seas,indic.1985,indic.1986,indic.1987,
              day.in.seas*indic.1985,
              day.in.seas*indic.1986,
              day.in.seas*indic.1987,TSP,temperature,rel.humid)
              

   # Add on splines for temp and rel.humid
   
   
   K.temp <- 15
   knots.temp <- quantile(unique(temperature),
                          seq(0,1,length=K.temp+2))[-c(1,K.temp+2)]
   
   K.relh <- 15
   knots.relh <- quantile(unique(rel.humid),
                          seq(0,1,length=K.relh+2))[-c(1,K.relh+2)]
   
   Z.temp <- outer(temperature,knots.temp,"-")
   Z.temp <- Z.temp*(Z.temp>0)
   
   Z.relh <- outer(rel.humid,knots.relh,"-")
   Z.relh <- Z.relh*(Z.relh>0)
   

   # Set up Z matrix
   
   Z.overall <- outer(day.in.seas,knots,"-")
   Z.overall <- Z.overall*(Z.overall>0)
   
   Z <- cbind(Z.overall,indic.1984*Z.overall,indic.1985*Z.overall,
              indic.1986*Z.overall,indic.1987*Z.overall)
   
   re.block.inds <- list(1:K,(K+1):(2*K),(2*K+1):(3*K),
                         (3*K+1):(4*K),(4*K+1):(5*K),
                         (5*K+1):(5*K+K.temp),
                         (5*K+K.temp+1):(5*K+K.temp+K.relh))

   Z <- cbind(Z,Z.temp,Z.relh)
                         
   Z.block  <-  list()
   for (i in 1:length(re.block.inds))
      Z.block[[i]] <- as.formula(paste("~Z[,c(",paste(
                re.block.inds[[i]],collapse=","),")]-1"))


   fit <- lme(sqrt.mort~-1+X,random=pdBlocked(Z.block,pdClass="pdIdent"))

   beta.hat <- fit$coef$fixed
   u.hat <- unlist(fit$coef$random)

   sig.sq.eps <- fit$sigma^2
   sig.sq.u   <- sig.sq.eps*exp(2*unlist(fit$modelStruct))
   
  

   # Plot for each year

   ng <- 101
   day.in.seas.grid <- seq(min(day.in.seas),max(day.in.seas),length=ng)
   
   par(mfrow=c(2,3))
   
   # Plot for 1984
   
   X.inds.1984 <- c(1,3)
   Z.inds.1984 <- c(1:(2*K))
   
   X.grid <- cbind(rep(1,ng),day.in.seas.grid)
    
   Z.grid <- outer(day.in.seas.grid,knots,"-")
   Z.grid <- Z.grid*(Z.grid>0)
   
   Z.grid <- cbind(Z.grid,Z.grid)
   
   f.hat.1984 <- X.grid%*%beta.hat[X.inds.1984] + Z.grid%*%u.hat[Z.inds.1984]
   

   # Plot for 1985
   
   X.grid <- cbind(X.grid,X.grid)
   
   X.inds.1985 <- c(1,3,4,7)
   Z.inds.1985 <- c(1:K,(2*K+1):(3*K))
   
   f.hat.1985 <- X.grid%*%beta.hat[X.inds.1985] + Z.grid%*%u.hat[Z.inds.1985]
   
   # Plot for 1986
   
   X.inds.1986 <- c(1,3,5,8)
   Z.inds.1986 <- c(1:K,(3*K+1):(4*K))
   
   f.hat.1986 <- X.grid%*%beta.hat[X.inds.1986] + Z.grid%*%u.hat[Z.inds.1986]
   
   # Plot for 1987
   
   X.inds.1987 <- c(1,3,6,9)
   Z.inds.1987 <- c(1:K,(4*K+1):(5*K))
   
   f.hat.1987 <- X.grid%*%beta.hat[X.inds.1987] + Z.grid%*%u.hat[Z.inds.1987]

   y.range <- range(c(f.hat.1984,f.hat.1985,f.hat.1986,f.hat.1987))
   
   plot(day.in.seas.grid,f.hat.1984,type="l",ylim=y.range)
   plot(day.in.seas.grid,f.hat.1985,type="l",ylim=y.range)
   plot(day.in.seas.grid,f.hat.1986,type="l",ylim=y.range)
   plot(day.in.seas.grid,f.hat.1987,type="l",ylim=y.range)

   # Plot for temperature

   temperature.grid <- seq(min(temperature),max(temperature),length=ng)

   X.inds.temp <- c(1,11)
   Z.inds.temp <- c((5*K+1):(5*K+K.temp))

   X.grid <- cbind(rep(1,ng),temperature.grid)
 
   Z.grid <- outer(temperature.grid,knots.temp,"-")
   Z.grid <- Z.grid*(Z.grid>0)
   f.hat.temp <- X.grid%*%beta.hat[X.inds.temp] + Z.grid%*%u.hat[Z.inds.temp]

   plot(temperature.grid,f.hat.temp,type="l")

   # Plot for relative humidity

   rel.humid.grid <- seq(min(rel.humid),max(rel.humid),length=ng)

   X.inds.relh <- c(1,12)
   Z.inds.relh <- c((5*K+K.temp+1):(5*K+K.temp+K.relh))

   X.grid <- cbind(rep(1,ng),rel.humid.grid)
 
   Z.grid <- outer(rel.humid.grid,knots.relh,"-")
   Z.grid <- Z.grid*(Z.grid>0)
   f.hat.relh <- X.grid%*%beta.hat[X.inds.relh] + Z.grid%*%u.hat[Z.inds.relh]

   plot(rel.humid.grid,f.hat.relh,type="l")



########## End of sec.7.1.s ##########

