/********** SAS code: sec.7.2.sas **************

 SAS code listed in Section 7.2 of
 the paper "Smoothing and Mixed Model Software"
 by Ngo & Wand
 
 ***********************************************/

libname paper '~/test';
data milan1;
  infile '~/test/milanmort.dat' missover;
  input daynum dayweek holiday temperature relhumid totmort respmort s02 tsp;
  sqrtmort=sqrt(respmort);
  if daynum ne . ;
run;
data paper.milan1;
  set milan1;
  if 1 <= _n_ <= 4*365; 
  
  if  1<= _n_ <= 365 then  do;
      indic1984=1; 
      dayinseas=_n_;
  end; else              
  if (365+1) <= _n_ <= 365*2 then do;
      indic1985=1;
      dayinseas=_n_-365;
  end; else 
  if (365*2+1) <= _n_ <= 365*3 then do;
      indic1986=1;
      dayinseas=_n_-365*2;
  end; else    
  if (365*3+1) <= _n_ <= 365*4 then do;
      indic1987=1;
      dayinseas=_n_-365*3;
  end; 
  
  
  array ind (4) indic1984-indic1987;
  do i=1 to 4;
     if ind(i)=. then ind(i)=0;
  end;
  m=1;
 run;    
        
%include "default_knots.macro"; 
%default_knots(librefknots=paper,data=paper.milan1,knotdata=knots1,varknots=temperature);
%default_knots(librefknots=paper,data=paper.milan1,knotdata=knots2,varknots=relhumid);
%default_knots(librefknots=paper,data=paper.milan1,knotdata=knots3,varknots=dayinseas);
data kt1;
  set paper.knots1 nobs=nk1;
  call symput('nkt1',nk1);
run;
proc transpose data=paper.knots1 prefix=knots1_ out=knotst1;
  var knots;
run;
data kt2;
  set paper.knots2 nobs=nk2;
  call symput('nkt2',nk2);
run;
proc transpose data=paper.knots2 prefix=knots2_ out=knotst2;
  var knots;
run;
data kt3;
  set paper.knots3 nobs=nk3;
  call symput('nkt3',nk3);
run;
proc transpose data=paper.knots3 prefix=knots3_ out=knotst3;
  var knots;
run;
data paper.knotst;
  merge knotst1 knotst2 knotst3;
  m=1;
run;
/***********************************/
/* creating the Z matrix           */
/***********************************/

data dataw;
  merge paper.milan1 paper.knotst;
  by m;
  %let nk1=&nkt1;
  %let nk2=&nkt2;
  %let nk3=&nkt3;
  array Z1a (&nk1) Z1_1-Z1_&nk1;
  array knots1a (&nk1) knots1_1-knots1_&nk1;
  do k=1 to &nk1;
     Z1a(k)=temperature-knots1a(k);
     if Z1a(k) < 0 then Z1a(k)=0;
  end;
  array Z2a (&nk2) Z2_1-Z2_&nk2;
  array knots2a (&nk2) knots2_1-knots2_&nk2;
  do k=1 to &nk2;
     Z2a(k)=relhumid-knots2a(k);
     if Z2a(k) < 0 then Z2a(k)=0;
  end;
  array Z3a (&nk3) Z3_1-Z3_&nk3;
  array knots3a (&nk3) knots3_1-knots3_&nk3;
  
  array intera1a  (&nk3) inter1_1-inter1_&nk3;
  array intera2a  (&nk3) inter2_1-inter2_&nk3;
  array intera3a  (&nk3) inter3_1-inter3_&nk3;
  array intera4a  (&nk3) inter4_1-inter4_&nk3;
     
  do k=1 to &nk3;
     Z3a(k)=dayinseas-knots3a(k);
     if Z3a(k) < 0 then Z3a(k)=0;
     *constructing interaction terms;
     intera1a(k) = indic1984*Z3a(k);
     intera2a(k) = indic1985*Z3a(k);
     intera3a(k) = indic1986*Z3a(k);
     intera4a(k) = indic1987*Z3a(k);
  end;
  drop knots1_1-knots1_&nk1 knots2_1-knots2_&nk2 knots3_1-knots3_&nk3 _name_;
run;


ods output CovParms=paper.varcomp;

/************************************/
/* fitting the additive model       */
/************************************/

proc mixed;
  model sqrtmort =  holiday dayinseas 
                    indic1985 indic1986 indic1987  
                    indic1985*dayinseas indic1986*dayinseas indic1987*dayinseas
                    tsp temperature relhumid
                    / solution outp=paper.yhat;   
  random  Z3_1-Z3_&nk3 / type=toep(1) s;
  random inter1_1-inter1_&nk3 / type=toep(1) s;
  random inter2_1-inter2_&nk3 / type=toep(1) s;
  random inter3_1-inter3_&nk3 / type=toep(1) s;
  random inter4_1-inter4_&nk3 / type=toep(1) s;
  random  Z1_1-Z1_&nk1 / type=toep(1) s;
  random  Z2_1-Z2_&nk2 / type=toep(1) s;
run;
                
************* End of sec.7.2.sas **************




