/********** SAS code: sec.8.2.sas **************

 SAS code listed in Section 8.2 of
 the paper "Smoothing and Mixed Model Software"
 by Ngo & Wand
 
 ***********************************************/

libname paper '~/test';
data paper.ethanol;
  infile '~/test/ethanol.dat' missover;
  input idnum nox c e;
  if idnum ne .; 
run;  

/*********************************************/
/* creating knots for the smoothing variable */
/*********************************************/

%include "default_knots.macro"; 
%default_knots(librefknots=paper,data=paper.ethanol,knotdata=knots,varknots=e);

data dataw;
  set paper.ethanol;
  m=1;
run;

data kt;
  set paper.knots nobs=nk;
  call symput('nkt',nk);
run;
proc transpose data=paper.knots prefix=knots out=knotst;
  var knots;
run;


data paper.knotst;
  set knotst;
  m=1;
run;

/***********************************/
/* creating the Z matrix           */
/***********************************/

data dataw;
  merge dataw paper.knotst;
  by m;
  %let nk=&nkt;
  array Z  (&nk) Z1-Z&nk;
  array XZ (&nk) XZ1-XZ&nk;
  array knots (&nk) knots1-knots&nk;
  do k=1 to &nk;
     Z(k)=e-knots(k);
     if Z(k) < 0 then Z(k)=0;
     XZ(k)=c*Z(k);
  end;
  drop knots1-knots&nk _name_;
run;

ods output CovParms=paper.varcomp;

/************************************/
/* fitting the additive model       */
/************************************/

proc mixed;
  model nox = e c e*c / solution ; *outp=paper.yhat;
  random  Z1-Z&nk  / type=toep(1) s;
  random XZ1-XZ&nk / type=toep(1) s;
run;

************* End of sec.8.2.sas ************** 
