########## S script: sec.9.2.s ##########

# S-PLUS code listed in Section 9.2 of
# the paper "Smoothing and Mixed Model Software"
# by Ngo & Wand.

# Load in default.knots() function.
	
source("default.knots.sf")	

# Read in the fossil data and assign scatterplot
# vectors to x and y.	
		
fossil <- read.table("fossil.dat",header=T)
x <- fossil$age
y <- 100000*fossil$strontium.ratio

# Set smoothing parameter (variance ratio).
		
alpha <- 2

# Set up design matrices, for linear splines
# in this case.
	
n <- length(x)
X <- cbind(rep(1,n),x)
knots <- default.knots(x)
Z <- outer(x,knots,"-")
Z <- Z*(Z>0)

# Set up input matrices for Algorithm 1.
	
C.mat <- cbind(X,Z)
D.mat <- diag(c(rep(0,ncol(X)),rep(1,ncol(Z))))

# Carry out Steps 1 and 2 of Algorithm 1.
		
svd.C <- svd(C.mat)
U.C <- svd.C$u
V.C <- svd.C$v
d.C <- svd.C$d
svd.D <- svd(t(t(t(V.C)%*%D.mat%*%V.C/d.C)/d.C))
d.D <- svd.D$d

# Obtain A matrix and b vector.

A.mat <- U.C%*%svd.D$u  
b.vec <- as.vector(t(A.mat)%*%y)

# Obtain vector of fitted values.
	
f.hat <- A.mat%*%(b.vec/(1+alpha*d.D))

# Obtain number of degrees of freedom fit.
	
df.fit <- sum(1/(1+alpha*d.D))

# Plot the fit.
		
plot(x,y,pch=1)
lines(x[order(x)],f.hat[order(x)],type="l")
	
########## End of sec.9.3.s ##########







