/********** SAS code: sec.9.3.sas **************

 SAS code listed in Section 9.3 of
 the paper "Smoothing and Mixed Model Software"
 by Ngo & Wand
 
 ***********************************************/

libname paper '~/test';
data paper.fossil;
  infile '~/test/fossil.dat' missover;
  input age ratio;
  ratio=ratio*100000;
  if age ne .;
run;  

/*********************************/
/* calling macro to create knots */
/*********************************/  

%include "default_knots.macro";
%default_knots(librefknots=paper,data=paper.fossil,knotdata=knots,varknots=age);
data dataw;
  set paper.fossil;
  m=1;
run;
data kt1;
  set paper.knots nobs=nk;
  call symput('nkt',nk);
run;
proc transpose data=paper.knots prefix=knots out=knotst;
  var knots;
run;
data paper.knotst;
  set knotst;
  m=1;
run;


/********************************/
/* creating the Z matrix        */
/********************************/ 

data dataw;
  merge dataw paper.knotst;
  by m;
  %let nk=&nkt;
  array Z (&nk) Z1-Z&nk;
  array knots (&nk) knots1-knots&nk;
  do k=1 to &nk;
     Z(k)=age-knots(k);
     if Z(k) < 0 then Z(k)=0;
  end;
  drop knots1-knots&nk _name_;
run;
ods output CovParms=paper.varcomp;

/********************************/
/* fitting the mixed model      */
/********************************/ 

proc mixed noprofile; *noprofile stops the algorithm from profiling;
		      *out the variance of the error term;
  model ratio = age / solution outp=paper.yhat;
  random  Z1-Z&nk / type=toep(1) s;
  *parms (400) (1)   / noiter; *noiter prevents Newton-Raphson iterative;
  			       *algorithm from changing variance components; 
  *parms (3.2) (2)   / noiter;
   parms (15)  (100) / noiter;
run;

/********************************/
/* plotting the smoothed curve  */
/********************************/
proc sort;
  by age;
run;
symbol1 v=circle c=black i=j l=1;
symbol2 v=point  c=blue  i=j l=2;
goptions device=xcolor;
proc gplot;
  plot ratio*age pred*age / overlay;
run;  
  
************* End of sec.9.4.sas **************  
